@@ -8,8 +8,6 @@
  *
  */
 
-#define WNCK_I_KNOW_THIS_IS_UNSTABLE 1
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -21,7 +19,6 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <glade/glade-xml.h>
 #include <libwnck/libwnck.h>
 #include <gconf/gconf-client.h>
 
@@ -60,15 +57,8 @@ typedef struct {
 	guint listeners [3];
 } TasklistData;
 
-static void display_properties_dialog (BonoboUIComponent *uic,
-				       TasklistData      *tasklist,
-				       const gchar       *verbname);
-static void display_help_dialog       (BonoboUIComponent *uic,
-				       TasklistData      *tasklist,
-				       const gchar       *verbname);
-static void display_about_dialog      (BonoboUIComponent *uic,
-				       TasklistData      *tasklist,
-				       const gchar       *verbname);
+static void display_properties_dialog (GtkAction    *action,
+				       TasklistData *tasklist);
 
 static void
 tasklist_update (TasklistData *tasklist)
@@ -136,23 +126,13 @@ applet_change_orient (PanelApplet       *applet,
 }
 
 static void
-applet_change_background (PanelApplet               *applet,
-			  PanelAppletBackgroundType  type,
-			  GdkColor                  *color,
-			  GdkPixmap                 *pixmap,
-			  TasklistData              *tasklist)
+applet_change_background (PanelApplet     *applet,
+			  cairo_pattern_t *pattern,
+			  TasklistData    *tasklist)
 {
-	switch (type) {
-	case PANEL_NO_BACKGROUND:
-		wnck_tasklist_set_button_relief (WNCK_TASKLIST (tasklist->tasklist),
-						 GTK_RELIEF_NORMAL);
-		break;
-	case PANEL_COLOR_BACKGROUND:
-	case PANEL_PIXMAP_BACKGROUND:
-		wnck_tasklist_set_button_relief (WNCK_TASKLIST (tasklist->tasklist),
-						 GTK_RELIEF_NONE);
-		break;
-	}
+        wnck_tasklist_set_button_relief (WNCK_TASKLIST (tasklist->tasklist),
+                                         pattern != NULL ? GTK_RELIEF_NONE
+                                                         : GTK_RELIEF_NORMAL);
 }
 
 static void
@@ -192,11 +172,10 @@ destroy_tasklist(GtkWidget * widget, TasklistData *tasklist)
         g_free (tasklist);
 }
 
-static const BonoboUIVerb tasklist_menu_verbs [] = {
-	BONOBO_UI_UNSAFE_VERB ("TasklistPreferences", display_properties_dialog),
-	BONOBO_UI_UNSAFE_VERB ("TasklistHelp",        display_help_dialog),
-	BONOBO_UI_UNSAFE_VERB ("TasklistAbout",       display_about_dialog),
-        BONOBO_UI_VERB_END
+static const GtkActionEntry tasklist_menu_actions [] = {
+	{ "TasklistPreferences", GTK_STOCK_PROPERTIES, N_("_Preferences"),
+	  NULL, NULL,
+	  G_CALLBACK (display_properties_dialog) }
 };
 
 static void
@@ -391,18 +370,14 @@ setup_gconf (TasklistData *tasklist)
 }
 
 static void
-applet_size_request (GtkWidget      *widget,
-		     GtkRequisition *requisition,
-		     TasklistData   *tasklist)
+applet_size_allocate (GtkWidget      *widget,
+                      GtkAllocation  *allocation,
+                      TasklistData   *tasklist)
 {
 	int len;
 	const int *size_hints;
-	GtkRequisition child_req;
-    	WnckTasklist *wncktl = WNCK_TASKLIST (tasklist->tasklist);
 
-	gtk_widget_get_child_requisition (tasklist->applet, &child_req);
-
-	size_hints = wnck_tasklist_get_size_hint_list (wncktl, &len);
+	size_hints = wnck_tasklist_get_size_hint_list (WNCK_TASKLIST (tasklist->tasklist), &len);
 	g_assert (len % 2 == 0);
 
         panel_applet_set_size_hints (PANEL_APPLET (tasklist->applet),
@@ -463,6 +438,8 @@ gboolean
 window_list_applet_fill (PanelApplet *applet)
 {
 	TasklistData *tasklist;
+	GtkActionGroup *action_group;
+	gchar *ui_path;
 	GError *error;
 	GConfValue *value;
 
@@ -518,7 +495,7 @@ window_list_applet_fill (PanelApplet *applet)
 		break;
 	}
 
-	tasklist->tasklist = wnck_tasklist_new (NULL);
+	tasklist->tasklist = wnck_tasklist_new ();
 
         wnck_tasklist_set_icon_loader (WNCK_TASKLIST (tasklist->tasklist),
                                        icon_loader_func,
@@ -528,9 +505,8 @@ window_list_applet_fill (PanelApplet *applet)
 	g_signal_connect (G_OBJECT (tasklist->tasklist), "destroy",
 			  G_CALLBACK (destroy_tasklist),
 			  tasklist);
-
-	g_signal_connect (G_OBJECT (tasklist->applet), "size_request",
-			  G_CALLBACK (applet_size_request),
+	g_signal_connect (G_OBJECT (tasklist->applet), "size_allocate",
+			  G_CALLBACK (applet_size_allocate),
 			  tasklist);
 	tasklist_update (tasklist);
 	gtk_widget_show (tasklist->tasklist);
@@ -556,68 +532,31 @@ window_list_applet_fill (PanelApplet *applet)
 
 	panel_applet_set_background_widget (PANEL_APPLET (tasklist->applet),
 					    GTK_WIDGET (tasklist->applet));
-	
+
+	action_group = gtk_action_group_new ("Tasklist Applet Actions");
+	gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE);
+	gtk_action_group_add_actions (action_group,
+				      tasklist_menu_actions,
+				      G_N_ELEMENTS (tasklist_menu_actions),
+				      tasklist);
+	ui_path = g_build_filename (WNCK_MENU_UI_DIR, "window-list-menu.xml", NULL);
 	panel_applet_setup_menu_from_file (PANEL_APPLET (tasklist->applet),
-					   NULL,
-					   "GNOME_WindowListApplet.xml",
-					   NULL,
-					   tasklist_menu_verbs, 
-					   tasklist);
+					   ui_path, action_group);
+	g_free (ui_path);
 
 	if (panel_applet_get_locked_down (PANEL_APPLET (tasklist->applet))) {
-		BonoboUIComponent *popup_component;
+		GtkAction *action;
 
-		popup_component = panel_applet_get_popup_component (PANEL_APPLET (tasklist->applet));
-
-		bonobo_ui_component_set_prop (popup_component,
-					      "/commands/TasklistPreferences",
-					      "hidden", "1",
-					      NULL);
+		action = gtk_action_group_get_action (action_group, "TasklistPreferences");
+		gtk_action_set_visible (action, FALSE);
 	}
+	g_object_unref (action_group);
 
 	gtk_widget_show (tasklist->applet);
 	
 	return TRUE;
 }
 
-
-static void
-display_help_dialog (BonoboUIComponent *uic,
-		     TasklistData      *tasklist,
-		     const gchar       *verbname)
-{
-	wncklet_display_help (tasklist->applet, "user-guide",
-			      "windowlist", WINDOW_LIST_ICON);
-}
-
-static void
-display_about_dialog (BonoboUIComponent *uic,
-		      TasklistData      *tasklist,
-		      const gchar       *verbname)
-{
-	static const gchar *authors[] =
-	{
-		"Alexander Larsson <alla@lysator.liu.se>",
-		NULL
-	};
-	const char *documenters [] = {
-		"Sun GNOME Documentation Team <gdocteam@sun.com>",
-		NULL
-	};
-	const char *translator_credits = _("translator-credits");
-
-	wncklet_display_about (tasklist->applet, &tasklist->about,
-			       _("Window List"),
-			       "Copyright \xc2\xa9 2001-2002 Red Hat, Inc.",
-			       _("The Window List shows a list of all windows in a set of buttons and lets you browse them."),
-			       authors,
-			       documenters,
-			       translator_credits,
-			       WINDOW_LIST_ICON,
-			       "tasklist",
-			       "Tasklist");
-}
-
 static void
 group_windows_toggled (GtkToggleButton *button,
 		       TasklistData    *tasklist)
@@ -649,10 +588,12 @@ display_all_workspaces_toggled (GtkToggleButton *button,
 				     NULL);
 }
 
+#define WID(s) GTK_WIDGET (gtk_builder_get_object (builder, s))
+
 static void
 setup_sensitivity (TasklistData *tasklist,
 		   GConfClient *client,
-		   GladeXML *xml,
+		   GtkBuilder *builder,
 		   const char *wid1,
 		   const char *wid2,
 		   const char *wid3,
@@ -670,27 +611,25 @@ setup_sensitivity (TasklistData *tasklist,
 	}
 	g_free (fullkey);
 
-	w = glade_xml_get_widget (xml, wid1);
+	w = WID (wid1);
 	g_assert (w != NULL);
 	gtk_widget_set_sensitive (w, FALSE);
 
 	if (wid2 != NULL) {
-		w = glade_xml_get_widget (xml, wid2);
+		w = WID (wid2);
 		g_assert (w != NULL);
 		gtk_widget_set_sensitive (w, FALSE);
 	}
 	if (wid3 != NULL) {
-		w = glade_xml_get_widget (xml, wid3);
+		w = WID (wid3);
 		g_assert (w != NULL);
 		gtk_widget_set_sensitive (w, FALSE);
 	}
 
 }
 
-#define WID(s) glade_xml_get_widget (xml, s)
-
 static void
-setup_dialog (GladeXML     *xml,
+setup_dialog (GtkBuilder   *builder,
 	      TasklistData *tasklist)
 {
 	GConfClient *client;
@@ -701,7 +640,7 @@ setup_dialog (GladeXML     *xml,
 	tasklist->show_current_radio = WID ("show_current_radio");
 	tasklist->show_all_radio = WID ("show_all_radio");
 
-	setup_sensitivity (tasklist, client, xml,
+	setup_sensitivity (tasklist, client, builder,
 			   "show_current_radio",
 			   "show_all_radio",
 			   NULL,
@@ -711,7 +650,7 @@ setup_dialog (GladeXML     *xml,
 	tasklist->auto_group_radio = WID ("auto_group_radio");
 	tasklist->always_group_radio = WID ("always_group_radio");
 
-	setup_sensitivity (tasklist, client, xml,
+	setup_sensitivity (tasklist, client, builder,
 			   "never_group_radio",
 			   "auto_group_radio",
 			   "always_group_radio",
@@ -721,7 +660,7 @@ setup_dialog (GladeXML     *xml,
 	tasklist->move_minimized_radio = WID ("move_minimized_radio");
 	tasklist->change_workspace_radio = WID ("change_workspace_radio");
 
-	setup_sensitivity (tasklist, client, xml,
+	setup_sensitivity (tasklist, client, builder,
 			   "move_minimized_radio",
 			   "change_workspace_radio",
 			   NULL,
@@ -767,22 +706,32 @@ setup_dialog (GladeXML     *xml,
 
 
 static void 
-display_properties_dialog (BonoboUIComponent *uic,
-			   TasklistData      *tasklist,
-			   const gchar       *verbname)
+display_properties_dialog (GtkAction    *action,
+			   TasklistData *tasklist)
 {
 	if (tasklist->properties_dialog == NULL) {
-		GladeXML  *xml;
+		GtkBuilder *builder;
+		GError     *error;
 
-		xml = glade_xml_new (TASKLIST_GLADEDIR "/window-list.glade", NULL, NULL);
-		tasklist->properties_dialog = glade_xml_get_widget (xml, "tasklist_properties_dialog");
+		builder = gtk_builder_new ();
+		gtk_builder_set_translation_domain (builder, GETTEXT_PACKAGE);
+
+		error = NULL;
+		gtk_builder_add_from_file (builder, TASKLIST_BUILDERDIR "/window-list.ui", &error);
+		if (error) {
+			g_warning ("Error loading preferences: %s", error->message);
+			g_error_free (error);
+			return;
+		}
+
+		tasklist->properties_dialog = WID ("tasklist_properties_dialog");
 
 		g_object_add_weak_pointer (G_OBJECT (tasklist->properties_dialog),
 					   (void**) &tasklist->properties_dialog);
 
-		setup_dialog (xml, tasklist);
+		setup_dialog (builder, tasklist);
 		
-		g_object_unref (G_OBJECT (xml));
+		g_object_unref (builder);
 	}
 
 	gtk_window_set_icon_name (GTK_WINDOW (tasklist->properties_dialog),
